/* Transistor Multiplexing 3x3 LEDs
   By: Marty Peltz
   Date: 7/19/2013
   Purpose: Control 9 LEDs in a 3x3 pattern using multiplexing.
            This program uses Timer 1 to set an interrupt every 1 second.
            Each second the display of the 3x3 LEDs change to a different pattern.
               Used in Martin Peltz's Instructiable
*/
/**** variables ****/
char col1_cathode = 0;
char col2_cathode = 1;
char col3_cathode = 2;
char row1_anode = 3;
char row2_anode = 4;
char row3_anode = 5;
// counter
char i = 0;
// matrix display array
char matrix_display[12] = {0x02, 0x05, 0x02, 0x07,0x00,0x07,0x05,0x05,0x05,0x07,0x07,0x07};
void setup()
{
  // Set Pins to outputs
  pinMode(col1_cathode, OUTPUT);
  pinMode(col2_cathode, OUTPUT);
  pinMode(col3_cathode, OUTPUT);
  pinMode(row1_anode, OUTPUT);
  pinMode(row2_anode, OUTPUT);
  pinMode(row3_anode, OUTPUT);
  // turn off all GPIO pins
  digitalWrite(col1_cathode, LOW);
  digitalWrite(col2_cathode, LOW);
  digitalWrite(col3_cathode, LOW);
  digitalWrite(row1_anode, LOW);
  digitalWrite(row2_anode, LOW);
  digitalWrite(row3_anode, LOW);
  // setup timer that goes off every 1 second
  timer1_init();
}
void loop()
{
  display_LED(matrix_display[i],(matrix_display[i + 1]),(matrix_display[i + 2]));
}
/****************************************************************
      Interrupt Vectors
****************************************************************/
// Timer Counter 1 interrupt
ISR(TIMER1_COMPA_vect)
{
  if((i + 3) == 12)
  {
    i = 0;
  }
  else
  {
    i = (i + 3); 
  }
}   
/****************************************************************
      Functions
****************************************************************/
/**** display LED function ****/
/* Purpose: Call this function with the hexdecimal parameter you wish to display
 * Notes: This function controls the Cathode, in my case controlling the columns
 */
void display_LED(char c1, char c2, char c3)
{
      row_anode(c1);                   // Call row_anode with first hexdecimal value 
    digitalWrite(col1_cathode, HIGH);    // Display the first column
    delay(1);                            // Wait a short delay so the LED is on for 1ms 
    digitalWrite(col1_cathode, LOW);     // disable first column before continuing
    row_anode(c2);                     // Call row_anode with second hexdecimal value
    digitalWrite(col2_cathode, HIGH);    // Display the second column
    delay(1);                            // Wait a short delay so the LED is on for 1ms
    digitalWrite(col2_cathode, LOW);     // disable second column before continuing
    row_anode(c3);                     // Call row_anode with second hexdecimal value
    digitalWrite(col3_cathode, HIGH);    // Display the third column
    delay(1);                            // Wait a short delay so the LED is on for 1ms
    digitalWrite(col3_cathode, LOW);     // disable third column before continuing
    
    // (note try changing the 1ms to 100ms-1s you will see a difference)
}
/**** Cathode (columns) function ****/
/* Purpose: Take paramater value and turn on related pins
 * Note: Due to using Arduinos digitalWrite you must set the pins,
 *        however if using DDRx/PORTx you can just apply a hexdecimal value to set the pins on a port.
 */
void row_anode(char LEDs_on)
{
  if(LEDs_on == 0x00)
  {
    digitalWrite(row1_anode, LOW); 
    digitalWrite(row2_anode, LOW);
    digitalWrite(row3_anode, LOW);
  }
  else if(LEDs_on == 0x01)
  {
    digitalWrite(row1_anode, HIGH); 
    digitalWrite(row2_anode, LOW);
    digitalWrite(row3_anode, LOW);
  }
  else if(LEDs_on == 0x02)
  {
    digitalWrite(row1_anode, LOW); 
    digitalWrite(row2_anode, HIGH);
    digitalWrite(row3_anode, LOW); 
  }
  else if(LEDs_on == 0x03)
  {
    digitalWrite(row1_anode, HIGH); 
    digitalWrite(row2_anode, HIGH);
    digitalWrite(row3_anode, LOW); 
  }
  else if(LEDs_on == 0x04)
  {
    digitalWrite(row1_anode, LOW); 
    digitalWrite(row2_anode, LOW);
    digitalWrite(row3_anode, HIGH); 
  }
  else if(LEDs_on == 0x05)
  {
    digitalWrite(row1_anode, HIGH); 
    digitalWrite(row2_anode, LOW);
    digitalWrite(row3_anode, HIGH); 
  }
  else if(LEDs_on == 0x06)
  {
    digitalWrite(row1_anode, LOW); 
    digitalWrite(row2_anode, HIGH);
    digitalWrite(row3_anode, HIGH); 
  }
  else if(LEDs_on == 0x07)
  {
    digitalWrite(row1_anode, HIGH); 
    digitalWrite(row2_anode, HIGH);
    digitalWrite(row3_anode, HIGH); 
  }
}

void timer1_init(void)
{
   // Setup Timer Interrupt
  cli();  // disable all interrupts
  //set timer1 interrupt at 1Hz
  TCCR1A = 0;  // set entire TCCR1A register to 0
  TCCR1B = 0;  // same for TCCR1B
  TCNT1  = 0;  //initialize counter value to 0
  // set compare match register for 1hz increments
  OCR1A = 15624;  // = (16*10^6) / (1*1024) - 1 (must be <65536)
  // turn on CTC mode
  TCCR1B |= (1 << WGM12);
  // Set CS12 and CS10 bits for 1024 prescaler
  TCCR1B |= (1 << CS12) | (1 << CS10);  
  // enable timer compare interrupt
  TIMSK1 |= (1 << OCIE1A);
  sei();  // enable all interrupts  
}
